import type { NodeKey, LexicalEditor, DOMExportOutput } from 'lexical';
import { DecoratorNode, EditorConfig } from 'lexical';


export class VariableNode extends DecoratorNode<JSX.Element> {

  __name: string;

  static getType(): string {
    return 'variable-node';
  }

  static clone(node: VariableNode): VariableNode {
    return new VariableNode(node.__name, node.__key);
  }

  constructor(name: string, key?: NodeKey) {
    super(key);
    this.__name = name;
  }

  setName(name: string) {
    const self = this.getWritable();
    this.__name = name;
  }

  getName(): string {
    const self = this.getLatest();
    return self.__name;
  }

  getTextContent(): string {
    const self = this.getLatest();
    return `{{${self.__name}}}`;
  }

  // View

  createDOM(config: EditorConfig): HTMLElement {
    const span = document.createElement("span");
    const theme = config.theme;
    const className = theme.image;
    if (className !== undefined) {
      span.className = `${className}`;
    }
    return span;
  }

  updateDOM(): false {
    return false;
  }

  exportDOM(editor: LexicalEditor): DOMExportOutput {
    const self = this.getLatest();

    return {
      element: new Text(self.getTextContent())
    };
  }

  decorate(): JSX.Element {
    return (
      <div className="variable-node">
        {this.getName()}
      </div>
    );
  }
}