import { useLexicalComposerContext } from "@lexical/react/LexicalComposerContext";
import {
  $copyNode,
  $getRoot,
  $isParagraphNode,
  $isElementNode,
  $getNodeByKey,
  $getSelection,
  type BaseSelection,
  $parseSerializedNode
} from "lexical";
import { Input } from '@graphpolaris/shared/lib/components/inputs';
import { VariableNode } from '../VariableNode';
import { useState } from 'react';
import { useGraphQueryResult } from '@graphpolaris/shared/lib/data-access';

export const InsertVariablesPlugin = () => {
  const [editor] = useLexicalComposerContext();

  const onChange = (value: string | number) => {
    editor.update(() => {
      const selection = $getSelection() as BaseSelection;
      
      const node = new VariableNode(String(value));
      
      selection.insertNodes([node]);

      // TODO: enable drag and dropping nodes
    });
  };

  const result = useGraphQueryResult();

  const nodeTypes = Object.keys(result.metaData?.nodes.types || {});
  function optionsForType(type: string) {
    const typeObj = result.metaData?.nodes.types[type] ?? {attributes: null};
    
    if (!('attributes' in typeObj) || typeObj.attributes == null) {
      return [];
    }
    

    return Object.entries(typeObj.attributes).map(([k,v]) => Object.keys(v).map(x => `${k}_${x}`)).flat();
  }

  return nodeTypes.map((nodeType) => 
      <Input
            type="dropdown"
            label={`${nodeType} variable`}
            value=""
            options={optionsForType(nodeType)}
            onChange={onChange}
            />
        );
};