import { useMemo, useState } from 'react';
import { NodeAttribute, QueryGraphEdges, SchemaReactflowEntityNode, SchemaReactflowRelationNode } from '../../model';
import { useAppDispatch, useQuerybuilderAttributesShown } from '../../..';
import { isEqual } from 'lodash-es';
import { PillAttributesItem } from './PillAttributesItem';

type PillAttributesProps = {
  node: SchemaReactflowEntityNode | SchemaReactflowRelationNode;
  attributes: NodeAttribute[];
  attributeEdges: (QueryGraphEdges | undefined)[];
  mr?: number;
  className?: string;
};

type IconMapType = {
  [key: string]: string | undefined;
};

const IconMap: IconMapType = {
  temporal: 'icon-[ic--baseline-calendar-today]',
  spatial: 'icon-[ic--baseline-map]',
  numerical: 'icon-[ic--baseline-numbers]',
  categorical: 'icon-[ic--baseline-abc]',
  default: 'icon-[ic--baseline-place]',
};

export const PillAttributes = (props: PillAttributesProps) => {
  const attributesBeingShown = useQuerybuilderAttributesShown();

  const attributesOfInterest = useMemo(() => {
    return props.attributes.map((attribute) =>
      attributesBeingShown.findIndex((x) => isEqual(x, attribute.handleData)) === -1 ? false : true,
    );
  }, [attributesBeingShown]);

  return (
    <div className={'border-[1px] border-secondary-200 divide-y divide-secondary-200 !z-50'}>
      {attributesOfInterest &&
        attributesOfInterest.map((showing, i) => {
          if (!showing) return null;
          return (
            <PillAttributesItem
              key={props.attributes[i].handleData.attributeName || i}
              node={props.node}
              attribute={props.attributes[i]}
              mr={props.mr}
              className={props.className}
              icon={
                props.attributes[i].handleData?.attributeDimension
                  ? IconMap[props.attributes[i].handleData.attributeDimension || 0]
                  : 'icon-[ic--outline-question-mark]'
              }
            />
          );
        })}
    </div>
  );
};
